#!/usr/bin/env python3
"""
SNMP Walk Tool
Parameters:
    - IP: Target IP address
    - port: Target port (default: 161)
Additional parameters in comments:
    - community: SNMP community string (default: public)
    - version: SNMP version (1 or 2c)
Requires: pysnmp package
"""
from pysnmp.hlapi import *
import argparse
from datetime import datetime

class SNMPWalker:
    def __init__(self, ip, port=161, community='public', version='2c'):
        self.ip = ip
        self.port = port
        self.community = community
        self.version = version
        self.oid_tree = {}
        
    def get_version(self):
        if self.version == '2c':
            return 1  # SNMPv2c
        return 0  # SNMPv1
        
    def walk(self):
        print(f"Starting SNMP walk on {self.ip}:{self.port}")
        print(f"Community: {self.community}")
        print(f"Version: {self.version}")
        print(f"Timestamp: {datetime.now()}\n")
        
        # Start with system MIB
        for (errorIndication,
             errorStatus,
             errorIndex,
             varBinds) in nextRequestObject(
                SnmpEngine(),
                CommunityData(self.community, mpModel=self.get_version()),
                UdpTransportTarget((self.ip, self.port)),
                ContextData(),
                ObjectType(ObjectIdentity('SNMPv2-MIB', 'system')),
                lexicographicMode=False):

            if errorIndication:
                print(f"Error: {errorIndication}")
                break
            elif errorStatus:
                print(f"Error: {errorStatus}")
                break
            else:
                for varBind in varBinds:
                    self.process_varbind(varBind)
                    
        print("\nSNMP Walk Results:")
        self.print_tree()
        
    def process_varbind(self, varBind):
        """Process and store SNMP variable bindings"""
        try:
            oid = str(varBind[0])
            value = str(varBind[1])
            
            # Store in tree structure
            parts = oid.split('.')
            current = self.oid_tree
            for part in parts[:-1]:
                if part not in current:
                    current[part] = {}
                current = current[part]
            current[parts[-1]] = value
            
            # Print immediate result
            print(f"OID: {oid}")
            print(f"Value: {value}")
            print("-" * 50)
            
        except Exception as e:
            print(f"Error processing SNMP response: {str(e)}")
            
    def print_tree(self, node=None, prefix='', level=0):
        """Print OID tree in a hierarchical format"""
        if node is None:
            node = self.oid_tree
            
        for key, value in node.items():
            if isinstance(value, dict):
                print(f"{prefix}├── {key}/")
                self.print_tree(value, prefix + "│   ", level + 1)
            else:
                print(f"{prefix}├── {key}: {value}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="SNMP Walk Tool")
    parser.add_argument("ip", help="Target IP address")
    parser.add_argument("--port", type=int, default=161,
                      help="Target port (default: 161)")
    parser.add_argument("--community", default="public",
                      help="SNMP community string (default: public)")
    parser.add_argument("--version", choices=['1', '2c'], default='2c',
                      help="SNMP version (default: 2c)")
    args = parser.parse_args()
    
    walker = SNMPWalker(args.ip, args.port, args.community, args.version)
    walker.walk()
